@@ -35,6 +35,7 @@
 #include <libgnomecanvas/gnome-canvas-polygon.h>
 #include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdkkeysyms.h>
 #include "gal/widgets/e-cursors.h"
 #include "gal/util/e-i18n.h"
 #include "gal/util/e-util.h"
@@ -94,14 +95,14 @@ static GdkPixmap *remove_col_pixmap, *remove_col_mask;
 static GdkPixmap *add_col_pixmap, *add_col_mask;
 
 enum {
-	ARG_0,
-	ARG_TABLE_HEADER,
-	ARG_FULL_HEADER,
-	ARG_DND_CODE,
-	ARG_TABLE_FONTSET,
-	ARG_SORT_INFO,
-	ARG_TABLE,
-	ARG_TREE
+	PROP_0,
+	PROP_TABLE_HEADER,
+	PROP_FULL_HEADER,
+	PROP_DND_CODE,
+	PROP_TABLE_FONTSET,
+	PROP_SORT_INFO,
+	PROP_TABLE,
+	PROP_TREE
 };
 
 enum {
@@ -115,7 +116,7 @@ static void scroll_off (ETableHeaderItem *ethi);
 static void scroll_on (ETableHeaderItem *ethi, guint scroll_direction);
 
 static void
-ethi_destroy (GtkObject *object){
+ethi_dispose (GObject *object){
 	ETableHeaderItem *ethi = E_TABLE_HEADER_ITEM (object);
 
 	ethi_drop_table_header (ethi);
@@ -129,23 +130,23 @@ ethi_destroy (GtkObject *object){
 
 	if (ethi->sort_info) {
 		if (ethi->sort_info_changed_id)
-			gtk_signal_disconnect (GTK_OBJECT(ethi->sort_info), ethi->sort_info_changed_id);
+			g_signal_handler_disconnect (ethi->sort_info, ethi->sort_info_changed_id);
 		if (ethi->group_info_changed_id)
-			gtk_signal_disconnect (GTK_OBJECT(ethi->sort_info), ethi->group_info_changed_id);
-		gtk_object_unref (GTK_OBJECT(ethi->sort_info));
+			g_signal_handler_disconnect (ethi->sort_info, ethi->group_info_changed_id);
+		g_object_unref (ethi->sort_info);
 		ethi->sort_info = NULL;
 	}
 
 	if (ethi->full_header)
-		gtk_object_unref (GTK_OBJECT(ethi->full_header));
+		g_object_unref (ethi->full_header);
 	ethi->full_header = NULL;
 
 	if (ethi->config)
-		gtk_object_destroy (GTK_OBJECT (ethi->config));
+		g_object_unref (ethi->config);
 	ethi->config = NULL;
 	
-	if (GTK_OBJECT_CLASS (ethi_parent_class)->destroy)
-		(*GTK_OBJECT_CLASS (ethi_parent_class)->destroy) (object);
+	if (G_OBJECT_CLASS (ethi_parent_class)->dispose)
+		(*G_OBJECT_CLASS (ethi_parent_class)->dispose) (object);
 }
 
 static int
@@ -170,7 +171,8 @@ e_table_header_item_get_height (ETableHeaderItem *ethi)
 		ETableCol *ecol = e_table_header_get_column (eth, col);
 		int height;
 
-		height = e_table_header_compute_height (ecol, style, ethi->font);
+		height = e_table_header_compute_height (ecol,
+							GTK_WIDGET (GNOME_CANVAS_ITEM (ethi)->canvas));
 
 		if (height > maxheight)
 			maxheight = height;
@@ -239,12 +241,12 @@ ethi_font_set (ETableHeaderItem *ethi, GdkFont *font)
 }
 
 static void
-ethi_font_load (ETableHeaderItem *ethi, char *fontname)
+ethi_font_load (ETableHeaderItem *ethi, const char *fontname)
 {
 	GdkFont *font = NULL;
 
 	if (fontname != NULL)
-	font = gdk_fontset_load (fontname);
+		font = gdk_fontset_load (fontname);
 
 	if (font == NULL) {
 		font = gtk_style_get_font (GTK_WIDGET (GNOME_CANVAS_ITEM (ethi)->canvas)->style);
@@ -258,16 +260,16 @@ ethi_font_load (ETableHeaderItem *ethi, char *fontname)
 static void
 ethi_drop_table_header (ETableHeaderItem *ethi)
 {
-	GtkObject *header;
+	GObject *header;
 	
 	if (!ethi->eth)
 		return;
 
-	header = GTK_OBJECT (ethi->eth);
-	gtk_signal_disconnect (header, ethi->structure_change_id);
-	gtk_signal_disconnect (header, ethi->dimension_change_id);
+	header = G_OBJECT (ethi->eth);
+	g_signal_handler_disconnect (header, ethi->structure_change_id);
+	g_signal_handler_disconnect (header, ethi->dimension_change_id);
 
-	gtk_object_unref (header);
+	g_object_unref (header);
 	ethi->eth = NULL;
 	ethi->width = 0;
 }
@@ -288,16 +290,16 @@ static void
 ethi_add_table_header (ETableHeaderItem *ethi, ETableHeader *header)
 {
 	ethi->eth = header;
-	gtk_object_ref (GTK_OBJECT (ethi->eth));
+	g_object_ref (ethi->eth);
 
 	ethi->height = e_table_header_item_get_height (ethi);
 
-	ethi->structure_change_id = gtk_signal_connect (
-		GTK_OBJECT (header), "structure_change",
-		GTK_SIGNAL_FUNC(structure_changed), ethi);
-	ethi->dimension_change_id = gtk_signal_connect (
-		GTK_OBJECT (header), "dimension_change",
-		GTK_SIGNAL_FUNC(dimension_changed), ethi);
+	ethi->structure_change_id = g_signal_connect (
+		header, "structure_change",
+		G_CALLBACK (structure_changed), ethi);
+	ethi->dimension_change_id = g_signal_connect (
+		header, "dimension_change",
+		G_CALLBACK (dimension_changed), ethi);
 	e_canvas_item_request_reflow(GNOME_CANVAS_ITEM(ethi));
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM(ethi));
 }
@@ -309,70 +311,73 @@ ethi_sort_info_changed (ETableSortInfo *sort_info, ETableHeaderItem *ethi)
 }
 
 static void
-ethi_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+ethi_set_property (GObject *object,
+		   guint prop_id,
+		   const GValue *value,
+		   GParamSpec *pspec)
 {
 	GnomeCanvasItem *item;
 	ETableHeaderItem *ethi;
 
-	item = GNOME_CANVAS_ITEM (o);
-	ethi = E_TABLE_HEADER_ITEM (o);
+	item = GNOME_CANVAS_ITEM (object);
+	ethi = E_TABLE_HEADER_ITEM (object);
 
-	switch (arg_id){
-	case ARG_TABLE_HEADER:
+	switch (prop_id){
+	case PROP_TABLE_HEADER:
 		ethi_drop_table_header (ethi);
-		ethi_add_table_header (ethi, E_TABLE_HEADER(GTK_VALUE_OBJECT (*arg)));
+		ethi_add_table_header (ethi, E_TABLE_HEADER(g_value_get_object (value)));
 		break;
 
-	case ARG_FULL_HEADER:
+	case PROP_FULL_HEADER:
 		if (ethi->full_header)
-			gtk_object_unref(GTK_OBJECT(ethi->full_header));
-		ethi->full_header = E_TABLE_HEADER(GTK_VALUE_OBJECT (*arg));
+			g_object_unref(ethi->full_header);
+		ethi->full_header = E_TABLE_HEADER(g_value_get_object (value));
 		if (ethi->full_header)
-			gtk_object_ref(GTK_OBJECT(ethi->full_header));
+			g_object_ref(ethi->full_header);
 		break;
 
-	case ARG_DND_CODE:
+	case PROP_DND_CODE:
 		g_free(ethi->dnd_code);
-		ethi->dnd_code = g_strdup (GTK_VALUE_STRING (*arg));
+		ethi->dnd_code = g_strdup (g_value_get_string (value));
 		break;
 
-	case ARG_TABLE_FONTSET:
-		ethi_font_load (ethi, GTK_VALUE_STRING (*arg));
+	case PROP_TABLE_FONTSET:
+		ethi_font_load (ethi, g_value_get_string (value));
 		break;
 
-	case ARG_SORT_INFO:
+	case PROP_SORT_INFO:
 		if (ethi->sort_info){
 			if (ethi->sort_info_changed_id)
-				gtk_signal_disconnect (
-					GTK_OBJECT(ethi->sort_info),
+				g_signal_handler_disconnect (
+					ethi->sort_info,
 					ethi->sort_info_changed_id);
 
 			if (ethi->group_info_changed_id)
-				gtk_signal_disconnect (
-					GTK_OBJECT(ethi->sort_info),
+				g_signal_handler_disconnect (
+					ethi->sort_info,
 					ethi->group_info_changed_id);
-			gtk_object_unref (GTK_OBJECT(ethi->sort_info));
+			g_object_unref (ethi->sort_info);
 		}
-		ethi->sort_info = GTK_VALUE_POINTER (*arg);
-		gtk_object_ref (GTK_OBJECT(ethi->sort_info));
+		ethi->sort_info = g_value_get_object (value);
+		g_object_ref (ethi->sort_info);
 		ethi->sort_info_changed_id =
-			gtk_signal_connect (
-				GTK_OBJECT(ethi->sort_info), "sort_info_changed",
-				GTK_SIGNAL_FUNC(ethi_sort_info_changed), ethi);
+			g_signal_connect (
+				ethi->sort_info, "sort_info_changed",
+				G_CALLBACK (ethi_sort_info_changed), ethi);
 		ethi->group_info_changed_id =
-			gtk_signal_connect (
-				GTK_OBJECT(ethi->sort_info), "group_info_changed",
-				GTK_SIGNAL_FUNC(ethi_sort_info_changed), ethi);
+			g_signal_connect (
+				ethi->sort_info, "group_info_changed",
+				G_CALLBACK(ethi_sort_info_changed), ethi);
 		break;
-	case ARG_TABLE:
-		if (GTK_VALUE_OBJECT(*arg))
-			ethi->table = E_TABLE(GTK_VALUE_OBJECT(*arg));
+	case PROP_TABLE:
+		if (g_value_get_object (value))
+			ethi->table = E_TABLE(g_value_get_object (value));
 		else
 			ethi->table = NULL;
 		break;
-	case ARG_TREE:
-		if (GTK_VALUE_OBJECT(*arg))
-			ethi->tree = E_TREE(GTK_VALUE_OBJECT(*arg));
+	case PROP_TREE:
+		if (g_value_get_object (value))
+			ethi->tree = E_TREE(g_value_get_object (value));
 		else
 			ethi->tree = NULL;
 		break;
@@ -381,21 +386,24 @@ ethi_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 }
 
 static void
-ethi_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+ethi_get_property (GObject *object,
+		   guint prop_id,
+		   GValue *value,
+		   GParamSpec *pspec)
 {
 	ETableHeaderItem *ethi;
 
-	ethi = E_TABLE_HEADER_ITEM (o);
+	ethi = E_TABLE_HEADER_ITEM (object);
 
-	switch (arg_id){
-	case ARG_FULL_HEADER:
-		GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (ethi->full_header);
+	switch (prop_id){
+	case PROP_FULL_HEADER:
+		g_value_set_object (value, ethi->full_header);
 		break;
-	case ARG_DND_CODE:
-		GTK_VALUE_STRING (*arg) = g_strdup (ethi->dnd_code);
+	case PROP_DND_CODE:
+		g_value_set_string (value, g_strdup (ethi->dnd_code));
 		break;
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
@@ -406,7 +414,7 @@ ethi_find_col_by_x (ETableHeaderItem *ethi, int x)
 	const int cols = e_table_header_count (ethi->eth);
 	int x1 = 0;
 	int col;
-
+	
 	d(g_print ("%s:%d: x = %d, x1 = %d\n", __FUNCTION__, __LINE__, x, x1));
 
 	x1 += ethi->group_indent_width;
@@ -591,7 +599,6 @@ moved (ETableHeaderItem *ethi, guint col, guint model_col)
 }
 #endif
 
-
 static void
 do_drag_motion(ETableHeaderItem *ethi,
 	       GdkDragContext *context,
@@ -599,10 +606,14 @@ do_drag_motion(ETableHeaderItem *ethi,
 	       gint y,
 	       guint time,
 	       gboolean recreate)
-  {
+{
+	d(g_print("In do_drag_motion\n"));
+	d(g_print("x = %d, y = %d, ethi->width = %d, ethi->height = %d\n", x, y, ethi->width, ethi->height));
+
 	if ((x >= 0) && (x <= (ethi->width)) &&
 	    (y >= 0) && (y <= (ethi->height))){
 		int col;
+		d(g_print("In header\n"));
 		
 		col = ethi_find_col_by_x_nearest (ethi, x);
 
@@ -686,25 +697,21 @@ static void
 context_destroyed (gpointer data)
 {
 	ETableHeaderItem *ethi = data;
-#ifndef NO_WARNINGS
-#warning FIXME
-#endif
-	/* if (!GTK_OBJECT_DESTROYED (ethi)) */
-	{
-		ethi->last_drop_x       = 0;
-		ethi->last_drop_y       = 0;
-		ethi->last_drop_time    = 0;
-		ethi->last_drop_context = NULL;
-		scroll_off (ethi);
-	}
-	gtk_object_unref (GTK_OBJECT (ethi));
+
+	ethi->last_drop_x       = 0;
+	ethi->last_drop_y       = 0;
+	ethi->last_drop_time    = 0;
+	ethi->last_drop_context = NULL;
+	scroll_off (ethi);
+
+	g_object_unref (ethi);
 }
 
 static void
 context_connect (ETableHeaderItem *ethi, GdkDragContext *context)
 {
 	if (g_dataset_get_data (context, "e-table-header-item") == NULL) {
-		gtk_object_ref (GTK_OBJECT (ethi));
+		g_object_ref (ethi);
 		g_dataset_set_data_full (context, "e-table-header-item", ethi, context_destroyed);
 	}
 }
@@ -719,7 +726,7 @@ ethi_drag_motion (GtkWidget *widget, GdkDragContext *context,
 
 	gdk_drag_status (context, 0, time);
 
-	droptype = gdk_atom_name (context->targets->data);
+	droptype = gdk_atom_name (GDK_POINTER_TO_ATOM (context->targets->data));
 	headertype = g_strdup_printf ("%s-%s", TARGET_ETABLE_COL_TYPE,
 				      ethi->dnd_code);
 
@@ -730,8 +737,7 @@ ethi_drag_motion (GtkWidget *widget, GdkDragContext *context,
 
 	g_free (headertype);
 
-	x -= widget->allocation.x;
-	y -= widget->allocation.y;
+	d(g_print ("y = %d, widget->allocation.y = %d, GTK_LAYOUT (widget)->vadjustment->value = %f\n", y, widget->allocation.y, GTK_LAYOUT (widget)->vadjustment->value));
 
 	if (x < 20)
 		direction |= ET_SCROLL_LEFT;
@@ -862,6 +868,7 @@ ethi_drag_drop (GtkWidget *canvas,
 		
 		if (col != -1) {
 			char *target = g_strdup_printf ("%s-%s", TARGET_ETABLE_COL_TYPE, ethi->dnd_code);
+			d(g_print ("ethi -  %s\n", target));
 			gtk_drag_get_data (canvas, context, gdk_atom_intern(target, FALSE), time);
 			g_free (target);
 		}
@@ -907,20 +914,20 @@ ethi_realize (GnomeCanvasItem *item)
   	g_free(ethi_drop_types[0].target); 
 
 	/* Drop signals */
-	ethi->drag_motion_id = gtk_signal_connect        (GTK_OBJECT (item->canvas), "drag_motion",
-							  GTK_SIGNAL_FUNC (ethi_drag_motion), ethi);
-	ethi->drag_leave_id = gtk_signal_connect         (GTK_OBJECT (item->canvas), "drag_leave",
-							  GTK_SIGNAL_FUNC (ethi_drag_leave), ethi);
-	ethi->drag_drop_id = gtk_signal_connect          (GTK_OBJECT (item->canvas), "drag_drop",
-							  GTK_SIGNAL_FUNC (ethi_drag_drop), ethi);
-	ethi->drag_data_received_id = gtk_signal_connect (GTK_OBJECT (item->canvas), "drag_data_received",
-							  GTK_SIGNAL_FUNC (ethi_drag_data_received), ethi);
+	ethi->drag_motion_id = g_signal_connect (item->canvas, "drag_motion",
+						 G_CALLBACK (ethi_drag_motion), ethi);
+	ethi->drag_leave_id = g_signal_connect (item->canvas, "drag_leave",
+						G_CALLBACK (ethi_drag_leave), ethi);
+	ethi->drag_drop_id = g_signal_connect (item->canvas, "drag_drop",
+					       G_CALLBACK (ethi_drag_drop), ethi);
+	ethi->drag_data_received_id = g_signal_connect (item->canvas, "drag_data_received",
+							G_CALLBACK (ethi_drag_data_received), ethi);
 
 	/* Drag signals */
-	ethi->drag_end_id = gtk_signal_connect           (GTK_OBJECT (item->canvas), "drag_end",
-							  GTK_SIGNAL_FUNC (ethi_drag_end), ethi);
-	ethi->drag_data_get_id = gtk_signal_connect      (GTK_OBJECT (item->canvas), "drag_data_get",
-							  GTK_SIGNAL_FUNC (ethi_drag_data_get), ethi);
+	ethi->drag_end_id = g_signal_connect (item->canvas, "drag_end",
+					      G_CALLBACK (ethi_drag_end), ethi);
+	ethi->drag_data_get_id = g_signal_connect (item->canvas, "drag_data_get",
+						   G_CALLBACK (ethi_drag_data_get), ethi);
 
 }
 
@@ -931,13 +938,13 @@ ethi_unrealize (GnomeCanvasItem *item)
 
 	gdk_font_unref (ethi->font);
 
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_motion_id);
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_leave_id);
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_drop_id);
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_data_received_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_motion_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_leave_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_drop_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_data_received_id);
 
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_end_id);
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_data_get_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_end_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_data_get_id);
 
 	gtk_drag_dest_unset (GTK_WIDGET (item->canvas));
 
@@ -969,8 +976,8 @@ ethi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width
 			g_hash_table_insert (arrows, 
 					     GINT_TO_POINTER (column.column),
 					     GINT_TO_POINTER (column.ascending ?
-							      E_TABLE_COL_ARROW_UP : 
-							      E_TABLE_COL_ARROW_DOWN));
+							      E_TABLE_COL_ARROW_DOWN : 
+							      E_TABLE_COL_ARROW_UP));
 		}
 		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
 		for (i = 0; i < length; i++) {
@@ -978,8 +985,8 @@ ethi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width
 			g_hash_table_insert (arrows, 
 					     GINT_TO_POINTER (column.column),
 					     GINT_TO_POINTER (column.ascending ?
-							      E_TABLE_COL_ARROW_UP : 
-							      E_TABLE_COL_ARROW_DOWN));
+							      E_TABLE_COL_ARROW_DOWN : 
+							      E_TABLE_COL_ARROW_UP));
 		}
 	}
 
@@ -1004,7 +1011,7 @@ ethi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width
 			continue;
 
 		e_table_header_draw_button (drawable, ecol,
-					    GTK_WIDGET (canvas)->style, ethi->font,
+					    GTK_WIDGET (canvas)->style,
 					    GTK_WIDGET_STATE (canvas),
 					    GTK_WIDGET (canvas),
 					    x1 - x, -y,
@@ -1055,6 +1062,8 @@ is_pointer_on_division (ETableHeaderItem *ethi, int pos, int *the_total, int *re
 
 			return TRUE;
 		}
+		if (return_col)
+			*return_col = col;
 
 		if (total > pos + TOLERANCE)
 			return FALSE;
@@ -1165,8 +1174,8 @@ ethi_start_drag (ETableHeaderItem *ethi, GdkEvent *event)
 				arrows, 
 				GINT_TO_POINTER (column.column),
 				GINT_TO_POINTER (column.ascending ?
-						 E_TABLE_COL_ARROW_UP : 
-						 E_TABLE_COL_ARROW_DOWN));
+						 E_TABLE_COL_ARROW_DOWN : 
+						 E_TABLE_COL_ARROW_UP));
 		}
 		length = e_table_sort_info_sorting_get_count(ethi->sort_info);
 		for (i = 0; i < length; i++) {
@@ -1178,8 +1187,8 @@ ethi_start_drag (ETableHeaderItem *ethi, GdkEvent *event)
 				arrows, 
 				GINT_TO_POINTER (column.column),
 				GINT_TO_POINTER (column.ascending ?
-						 E_TABLE_COL_ARROW_UP : 
-						 E_TABLE_COL_ARROW_DOWN));
+						 E_TABLE_COL_ARROW_DOWN : 
+						 E_TABLE_COL_ARROW_UP));
 		}
 	}
 
@@ -1196,7 +1205,7 @@ ethi_start_drag (ETableHeaderItem *ethi, GdkEvent *event)
 
 	e_table_header_draw_button (
 		pixmap, ecol,
-		widget->style, ethi->font,
+		widget->style,
 		GTK_WIDGET_STATE (widget),
 		widget,
 		0, 0,
@@ -1369,11 +1378,11 @@ static void
 ethi_popup_field_chooser(GtkWidget *widget, EthiHeaderInfo *info)
 {
 	GtkWidget *etfcd = e_table_field_chooser_dialog_new();
-	gtk_object_set(GTK_OBJECT(etfcd),
-		       "full_header", info->ethi->full_header,
-		       "header", info->ethi->eth,
-		       "dnd_code", info->ethi->dnd_code,
-		       NULL);
+	g_object_set(etfcd,
+		     "full_header", info->ethi->full_header,
+		     "header", info->ethi->eth,
+		     "dnd_code", info->ethi->dnd_code,
+		     NULL);
 	gtk_widget_show(etfcd);
 }
 
@@ -1388,9 +1397,9 @@ ethi_popup_best_fit(GtkWidget *widget, EthiHeaderInfo *info)
 	ETableHeaderItem *ethi = info->ethi;
 	int width;
 
-	gtk_signal_emit_by_name (GTK_OBJECT (ethi->eth),
-				 "request_width",
-				 info->col, &width);
+	g_signal_emit_by_name (ethi->eth,
+			       "request_width",
+			       info->col, &width);
 	/* Add 10 to stop it from "..."ing */
 	e_table_header_set_size (ethi->eth, info->col, width + 10);
 	
@@ -1404,8 +1413,9 @@ ethi_popup_format_columns(GtkWidget *widget, EthiHeaderInfo *info)
 }
 
 static void
-config_destroyed (GtkObject *object, ETableHeaderItem *ethi)
+config_destroyed (gpointer data, GObject *where_object_was)
 {
+	ETableHeaderItem *ethi = data;
 	ethi->config = NULL;
 }
 
@@ -1419,6 +1429,9 @@ apply_changes (ETableConfig *config, ETableHeaderItem *ethi)
 	if (ethi->tree)
 		e_tree_set_state (ethi->tree, state);
 	g_free (state);
+
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (config->dialog_toplevel),
+					   GTK_RESPONSE_APPLY, FALSE);
 }
 
 static void
@@ -1442,13 +1455,12 @@ ethi_popup_customize_view(GtkWidget *widget, EthiHeaderInfo *info)
 
 		ethi->config = e_table_config_new (
 				_("Customize Current View"),
-				spec, state);
-		gtk_signal_connect (
-			GTK_OBJECT (ethi->config), "destroy",
-			GTK_SIGNAL_FUNC (config_destroyed), ethi);
-		gtk_signal_connect (
-			GTK_OBJECT (ethi->config), "changed",
-			GTK_SIGNAL_FUNC (apply_changes), ethi);
+				spec, state, NULL);
+		g_object_weak_ref (G_OBJECT (ethi->config),
+				   config_destroyed, ethi);
+		g_signal_connect (
+			ethi->config, "changed",
+			G_CALLBACK (apply_changes), ethi);
 	}
 }
 
@@ -1462,21 +1474,21 @@ free_popup_info (GtkWidget *w, EthiHeaderInfo *info)
 /* Bit 2 is disabled if not "sortable". */
 /* Bit 4 is disabled if we don't have a pointer to our table object. */
 static EPopupMenu ethi_context_menu [] = {
-	E_POPUP_ITEM (N_("Sort Ascending"),            GTK_SIGNAL_FUNC(ethi_popup_sort_ascending),  2),
-	E_POPUP_ITEM (N_("Sort Descending"),           GTK_SIGNAL_FUNC(ethi_popup_sort_descending), 2),
-	E_POPUP_ITEM (N_("Unsort"),                    GTK_SIGNAL_FUNC(ethi_popup_unsort),          0),
+	E_POPUP_ITEM (N_("Sort Ascending"),            G_CALLBACK(ethi_popup_sort_ascending),  2),
+	E_POPUP_ITEM (N_("Sort Descending"),           G_CALLBACK(ethi_popup_sort_descending), 2),
+	E_POPUP_ITEM (N_("Unsort"),                    G_CALLBACK(ethi_popup_unsort),          0),
 	E_POPUP_SEPARATOR,
-	E_POPUP_ITEM (N_("Group By This Field"),       GTK_SIGNAL_FUNC(ethi_popup_group_field),     16),
-	E_POPUP_ITEM (N_("Group By Box"),              GTK_SIGNAL_FUNC(ethi_popup_group_box),       128),
+	E_POPUP_ITEM (N_("Group By This Field"),       G_CALLBACK(ethi_popup_group_field),     16),
+	E_POPUP_ITEM (N_("Group By Box"),              G_CALLBACK(ethi_popup_group_box),       128),
 	E_POPUP_SEPARATOR,
-	E_POPUP_ITEM (N_("Remove This Column"),        GTK_SIGNAL_FUNC(ethi_popup_remove_column),   8),
-	E_POPUP_ITEM (N_("Add a Column..."),           GTK_SIGNAL_FUNC(ethi_popup_field_chooser),   0),
+	E_POPUP_ITEM (N_("Remove This Column"),        G_CALLBACK(ethi_popup_remove_column),   8),
+	E_POPUP_ITEM (N_("Add a Column..."),           G_CALLBACK(ethi_popup_field_chooser),   0),
 	E_POPUP_SEPARATOR,
-	E_POPUP_ITEM (N_("Alignment"),                 GTK_SIGNAL_FUNC(ethi_popup_alignment),       128),
-	E_POPUP_ITEM (N_("Best Fit"),                  GTK_SIGNAL_FUNC(ethi_popup_best_fit),        2),
-	E_POPUP_ITEM (N_("Format Columns..."),         GTK_SIGNAL_FUNC(ethi_popup_format_columns),  128),
+	E_POPUP_ITEM (N_("Alignment"),                 G_CALLBACK(ethi_popup_alignment),       128),
+	E_POPUP_ITEM (N_("Best Fit"),                  G_CALLBACK(ethi_popup_best_fit),        2),
+	E_POPUP_ITEM (N_("Format Columns..."),         G_CALLBACK(ethi_popup_format_columns),  128),
 	E_POPUP_SEPARATOR,
-	E_POPUP_ITEM (N_("Customize Current View..."), GTK_SIGNAL_FUNC(ethi_popup_customize_view),  4),
+	E_POPUP_ITEM (N_("Customize Current View..."), G_CALLBACK(ethi_popup_customize_view),  4),
 	E_POPUP_TERMINATOR
 };
 
@@ -1497,29 +1509,28 @@ ethi_header_context_menu (ETableHeaderItem *ethi, GdkEventButton *event)
 						 ((e_table_header_count (ethi->eth) > 1) ? 0 : 8),
 						 ((e_table_sort_info_get_can_group (ethi->sort_info)) ? 0 : 16) +
 						 128, info, E_I18N_DOMAIN);
-	gtk_signal_connect (GTK_OBJECT (popup), "selection-done",
-			    GTK_SIGNAL_FUNC (free_popup_info), info);
+	g_object_ref (popup);
+	gtk_object_sink (GTK_OBJECT (popup));
+	g_signal_connect (popup, "selection-done",
+			  G_CALLBACK (free_popup_info), info);
 	e_popup_menu (popup, (GdkEvent *) event);
 }
 
 static void
 ethi_button_pressed (ETableHeaderItem *ethi, GdkEventButton *event)
 {
-	gtk_signal_emit (GTK_OBJECT (ethi),
-			 ethi_signals [BUTTON_PRESSED], event);
+	g_signal_emit (ethi,
+		       ethi_signals [BUTTON_PRESSED], 0, event);
 }
 
 static void
-ethi_change_sort_state (ETableHeaderItem *ethi, gdouble x)
+ethi_change_sort_state (ETableHeaderItem *ethi, ETableCol *col)
 {
-	ETableCol *col;
 	int model_col;
 	int length;
 	int i;
 	int found = FALSE;
 	
-	col = e_table_header_get_column (ethi->eth, ethi_find_col_by_x (ethi, x));
-
 	if (col == NULL)
 		return;
 
@@ -1656,6 +1667,9 @@ ethi_event (GnomeCanvasItem *item, GdkEvent *e)
 				ethi->click_x = e->button.x;
 				ethi->click_y = e->button.y;
 				ethi->maybe_drag = TRUE;
+				is_pointer_on_division (ethi, x, &start, &col);
+				ethi->selected_col = col;
+				e_canvas_item_grab_focus (item, TRUE);
 			} else if (e->button.button == 3){
 				ethi_header_context_menu (ethi, &e->button);
 			} else
@@ -1671,9 +1685,9 @@ ethi_event (GnomeCanvasItem *item, GdkEvent *e)
 			break;
 		else {
 			int width = 0;
-			gtk_signal_emit_by_name (GTK_OBJECT (ethi->eth),
-						 "request_width",
-						 (int)ethi->resize_col, &width);
+			g_signal_emit_by_name (ethi->eth,
+					       "request_width",
+					       (int)ethi->resize_col, &width);
 			/* Add 10 to stop it from "..."ing */
 			e_table_header_set_size (ethi->eth, ethi->resize_col, width + 10);
 
@@ -1692,14 +1706,65 @@ ethi_event (GnomeCanvasItem *item, GdkEvent *e)
 		if (ethi->resize_col != -1){
 			needs_ungrab = (ethi->resize_guide != NULL);
 			ethi_end_resize (ethi);
-		} else if (was_maybe_drag && ethi->sort_info) 
-			ethi_change_sort_state (ethi, e->button.x);
+		} else if (was_maybe_drag && ethi->sort_info) {
+			ETableCol *col;
+		
+			col = e_table_header_get_column (ethi->eth, ethi_find_col_by_x (ethi, e->button.x));
+			ethi_change_sort_state (ethi, col);
+		}
 		
 		if (needs_ungrab)
 			gnome_canvas_item_ungrab (item, e->button.time);
 
 		break;
 	}
+	case GDK_KEY_PRESS:
+		if ((e->key.keyval == GDK_F10) && (e->key.state & GDK_SHIFT_MASK)) {
+			EthiHeaderInfo *info = g_new(EthiHeaderInfo, 1);
+			ETableCol *col;
+			GtkMenu *popup;
+ 
+			info->ethi = ethi;
+			info->col = ethi->selected_col;
+			col = e_table_header_get_column (ethi->eth, info->col);
+			
+			popup = e_popup_menu_create_with_domain (ethi_context_menu,
+								 1 +
+								 (col->sortable ? 0 : 2) +
+								 ((ethi->table || ethi->tree) ? 0 : 4) + 
+								 ((e_table_header_count (ethi->eth) > 1) ? 0 : 8),
+								 ((e_table_sort_info_get_can_group (ethi->sort_info)) ? 0 : 16) +
+								 128, info, E_I18N_DOMAIN);
+			g_object_ref (popup);
+			gtk_object_sink (GTK_OBJECT (popup));
+			g_signal_connect (popup, "selection-done",
+					  G_CALLBACK (free_popup_info), info);
+			e_popup_menu (popup, NULL);
+		} else if (e->key.keyval == GDK_space) {
+			ETableCol *col;
+			
+			col = e_table_header_get_column (ethi->eth, ethi->selected_col);
+			ethi_change_sort_state (ethi, col);
+		} else if ((e->key.keyval == GDK_Right) || (e->key.keyval == GDK_KP_Right)) {
+			ETableCol *col;
+
+			if ((ethi->selected_col < 0) || (ethi->selected_col >= ethi->eth->col_count - 1))
+				ethi->selected_col = 0;
+			else 
+				ethi->selected_col++;
+			col = e_table_header_get_column (ethi->eth, ethi->selected_col);
+			ethi_change_sort_state (ethi, col);
+		} else if ((e->key.keyval == GDK_Left) || (e->key.keyval == GDK_KP_Left)) {
+			ETableCol *col;
+
+			if ((ethi->selected_col <= 0) || (ethi->selected_col >= ethi->eth->col_count))
+				ethi->selected_col = ethi->eth->col_count - 1;
+			else 
+				ethi->selected_col--;
+			col = e_table_header_get_column (ethi->eth, ethi->selected_col);
+			ethi_change_sort_state (ethi, col);
+		}
+		break;
 	
 	default:
 		return FALSE;
@@ -1708,15 +1773,15 @@ ethi_event (GnomeCanvasItem *item, GdkEvent *e)
 }
 
 static void
-ethi_class_init (GtkObjectClass *object_class)
+ethi_class_init (GObjectClass *object_class)
 {
 	GnomeCanvasItemClass *item_class = (GnomeCanvasItemClass *) object_class;
 
-	ethi_parent_class = gtk_type_class (PARENT_OBJECT_TYPE);
+	ethi_parent_class = g_type_class_ref (PARENT_OBJECT_TYPE);
 	
-	object_class->destroy = ethi_destroy;
-	object_class->set_arg = ethi_set_arg;
-	object_class->get_arg = ethi_get_arg;
+	object_class->dispose = ethi_dispose;
+	object_class->set_property = ethi_set_property;
+	object_class->get_property = ethi_get_property;
 
 	item_class->update      = ethi_update;
 	item_class->realize     = ethi_realize;
@@ -1724,21 +1789,55 @@ ethi_class_init (GtkObjectClass *object_class)
 	item_class->draw        = ethi_draw;
 	item_class->point       = ethi_point;
 	item_class->event       = ethi_event;
-	
-	gtk_object_add_arg_type ("ETableHeaderItem::ETableHeader", GTK_TYPE_OBJECT,
-				 GTK_ARG_WRITABLE, ARG_TABLE_HEADER);
-	gtk_object_add_arg_type ("ETableHeaderItem::full_header", GTK_TYPE_OBJECT,
-				 GTK_ARG_READWRITE, ARG_FULL_HEADER);
-	gtk_object_add_arg_type ("ETableHeaderItem::dnd_code", GTK_TYPE_STRING,
-				 GTK_ARG_READWRITE, ARG_DND_CODE);
-	gtk_object_add_arg_type ("ETableHeaderItem::fontset", GTK_TYPE_STRING,
-				 GTK_ARG_WRITABLE, ARG_TABLE_FONTSET);
-	gtk_object_add_arg_type ("ETableHeaderItem::sort_info", GTK_TYPE_OBJECT,
-				 GTK_ARG_WRITABLE, ARG_SORT_INFO);
-	gtk_object_add_arg_type ("ETableHeaderItem::table", GTK_TYPE_OBJECT,
-				 GTK_ARG_WRITABLE, ARG_TABLE);
-	gtk_object_add_arg_type ("ETableHeaderItem::tree", E_TREE_TYPE,
-				 GTK_ARG_WRITABLE, ARG_TREE);
+
+	g_object_class_install_property (object_class, PROP_DND_CODE,
+					 g_param_spec_string ("dnd_code",
+							      _("DnD code"),
+							      /*_( */"XXX blurb" /*)*/,
+							      NULL,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_TABLE_FONTSET,
+					 g_param_spec_string ("fontset",
+							      _("Fontset"),
+							      /*_( */"XXX blurb" /*)*/,
+							      NULL,
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class, PROP_FULL_HEADER,
+					 g_param_spec_object ("full_header",
+							      _("Full Header"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TABLE_HEADER_TYPE,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_TABLE_HEADER,
+					 g_param_spec_object ("ETableHeader",
+							      _("Header"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TABLE_HEADER_TYPE,
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class, PROP_SORT_INFO,
+					 g_param_spec_object ("sort_info",
+							      _("Sort Info"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TABLE_SORT_INFO_TYPE,
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class, PROP_TABLE,
+					 g_param_spec_object ("table",
+							      _("Table"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TABLE_TYPE,
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class, PROP_TREE,
+					 g_param_spec_object ("tree",
+							      _("Tree"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TREE_TYPE,
+							      G_PARAM_WRITABLE));
 
 	/*
 	 * Create our pixmaps for DnD
@@ -1753,14 +1852,13 @@ ethi_class_init (GtkObjectClass *object_class)
 		&add_col_mask, NULL, add_col_xpm);
 
 	ethi_signals [BUTTON_PRESSED] =
-		gtk_signal_new ("button_pressed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ETableHeaderItemClass, button_pressed),
-				e_marshal_NONE__BOXED,
-				GTK_TYPE_NONE, 1, GDK_TYPE_EVENT);
-		
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, ethi_signals, LAST_SIGNAL);
+		g_signal_new ("button_pressed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETableHeaderItemClass, button_pressed),
+			      NULL, NULL,
+			      e_marshal_NONE__BOXED,
+			      G_TYPE_NONE, 1, GDK_TYPE_EVENT);
 }
 
 static void
@@ -1786,28 +1884,13 @@ ethi_init (GnomeCanvasItem *item)
 	ethi->group_indent_width = 0;
 	ethi->table = NULL;
 	ethi->tree = NULL;
+
+	ethi->selected_col = 0;
 }
 
-GtkType
-e_table_header_item_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"ETableHeaderItem",
-			sizeof (ETableHeaderItem),
-			sizeof (ETableHeaderItemClass),
-			(GtkClassInitFunc) ethi_class_init,
-			(GtkObjectInitFunc) ethi_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (PARENT_OBJECT_TYPE, &info);
-	}
-
-	return type;
-}
-
+E_MAKE_TYPE (e_table_header_item,
+	     "ETableHeaderItem",
+	     ETableHeaderItem,
+	     ethi_class_init,
+	     ethi_init,
+	     PARENT_OBJECT_TYPE)
